package io.openems.edge.battery.bydcommercial;

import org.osgi.service.metatype.annotations.AttributeDefinition;
import org.osgi.service.metatype.annotations.ObjectClassDefinition;

import io.openems.edge.common.startstop.StartStopConfig;

@ObjectClassDefinition(//
		name = "Battery BYD Battery-Box Commercial C130", //
		description = "Implements the Byd Commercial C130 battery rack system.")
public @interface Config {

	/**
	 * The id.
	 * @return String
	 */
	@AttributeDefinition(name = "Component-ID", description = "Unique ID of this Component")
	String id() default "battery0";

	/**
	 * The alias.
	 * @return String
	 */
	@AttributeDefinition(name = "Alias", description = "Human-readable name of this Component; defaults to Component-ID")
	String alias() default "";

	/**
	 * The enabled.
	 * @return boolean
	 */
	@AttributeDefinition(name = "Is enabled?", description = "Is this Component enabled?")
	boolean enabled() default true;

	/**
	 * The startStop.
	 * @return StartStopConfig
	 */
	@AttributeDefinition(name = "Start/stop behaviour?", description = "Should this Component be forced to start or stop?")
	StartStopConfig startStop() default StartStopConfig.AUTO;

	/**
	 * The modbus_id.
	 * @return String
	 */
	@AttributeDefinition(name = "Modbus-ID", description = "ID of Modbus bridge.")
	String modbus_id() default "modbus0";

	/**
	 * The modbusUnitId.
	 * @return int
	 */
	@AttributeDefinition(name = "Modbus Unit-ID", description = "The Unit-ID of the Modbus device.")
	int modbusUnitId() default 1;

	/**
	 * The numberOfSlaves.
	 * @return int
	 */
	@AttributeDefinition(name = "Number of slaves", description = "The number of slaves in this battery rack (max. 20)", min = "1", max = "20")
	int numberOfSlaves() default 20;

	/**
	 * The Modbus_target.
	 * @return String
	 */
	@AttributeDefinition(name = "Modbus target filter", description = "This is auto-generated by 'Modbus-ID'.")
	String Modbus_target() default "";

	/**
	 * The webconsole_configurationFactory_nameHint.
	 * @return String
	 */
	String webconsole_configurationFactory_nameHint() default "Battery BYD Battery-Box Commercial C130 [{id}]";
}